package spi_slave::fifo;
use europa_module_factory;
@ISA = qw(europa_module_factory);
use strict;

sub add_contents_to_module
{
  my $this = shift;
  my $module = $this->module();

  my $dw = $this->datawidth();

  $module->add_contents(
    e_port->new({name => "read", direction => "input",}),
    e_port->new({name => "write", direction => "input",}),
    e_port->new({name => "full", direction => "output",}),
    e_port->new({name => "empty", direction => "output",}),
    e_port->new({name => "writedata", width => $dw, direction => "input",}),
    e_port->new({name => "readdata", width => $dw, direction => "output",}),
  );

  $module->add_contents(
    e_register->new({
      out => "readdata",
      in => "writedata",
      enable => "write",
    }),
    #  empty  read  write  p1_empty
    #      0     0      0         0
    #      0     0      1         0 (overflow)
    #      0     1      0         1
    #      0     1      1         0
    #      1     0      0         1
    #      1     0      1         0
    #      1     1      0         1 (underflow)
    #      1     1      1         0 (underflow)
    e_assign->new({
      lhs => "p1_empty",
      rhs => "(read & ~write) | (empty & ~read & ~write)",
    }),
    e_register->new({
      out => "empty",
      in => "p1_empty",
      enable => "1'b1",
      async_value => "1'b1",
    }),
    #   full  read  write  p1_full
    #      0     0      0        0
    #      0     0      1        1
    #      0     1      0        0 (underflow)
    #      0     1      1        1 (underflow)
    #      1     0      0        1
    #      1     0      1        1 (overflow)
    #      1     1      0        0
    #      1     1      1        1

    # Naturally (since I have only a single storage location),
    # full = ~empty.  I'll encode that fact in a slightly more
    # elaborate way, to preserve the sort of structure I'm going
    # to need if/when this FIFO ever has more than one location.
    e_register->new({
      out => "full",
      in => "~p1_empty",
      enable => "1'b1",
      async_value => "1'b0",
    }),
  );

}

sub get_fields
{
  my $class = shift;

  my %fields = (
    datawidth => 
    {
      range => [1, undef],
    },
  );

  return \%fields;
}

1;

